/*========================================================
// HMD-01 High Power DC Motor Driver
// MCU: AT89S52
// Crystal: 12MHz
// Using the timer control variable duty cycle of the PWM pulse signal
// Occupied hardware resources T0,
// Through the key KEY1, KEY2, KEY3, KEY4 control
// PWM = 0% ~ `100%
// 1K frequency PWM cycle 1ms
// PWM output active low (duty xxx%)
// Note: Before energizing the motor module to check the wiring is correct before power.
========================================================*/
#include<reg52.h>

#define uchar unsigned char
#define uint  unsigned int

//===========Input keys========================

sbit  KEY_M1_SW     = P1^4;		 //M1:Start or stop
sbit  KEY_M1_DR     = P1^5;		 //M1:Forward or reverse
sbit  KEY_M1_ADD    = P1^6;		 //M1: Plus a PWM
sbit  KEY_M1_SUB    = P1^7;		 //M1:PWM minus one

//===========Output control pin========================							

sbit  MOTOR1_DR1  = P3^0;		 //M1:Electric steering control 1
sbit  MOTOR1_DR2  = P3^1;	 	 //M1:Electric steering control 2
sbit  PWM1_OUT    = P3^4;	     //M1:PWM output pin
sbit  BEEP        = P3^7 ;       //Buzzer



uchar PWM1_value  = 50;          //Initial value of 50%
uchar KeySpeedup =0;             //Key Acceleration

//Key signs
bdata uchar KEY_SIGN=0;

sbit KEY_M1_SW_S  = KEY_SIGN^0; //Key state
sbit KEY_M1_DR_S  = KEY_SIGN^1;	// Key state

//Motor signs
bdata unsigned char MOTOR_SIGN=0;
sbit MOTOR1_SW_S       = MOTOR_SIGN^0; //DC motor start = 1; Stop = 0; logo
sbit MOTOR1_DR_S       = MOTOR_SIGN^1; //DC motor rotational direction, forward = 0; Reverse = 1;
sbit MOTOR1_PWM_S      = MOTOR_SIGN^2; //PWM1 output high = 1; PWM1 output low = 0;


/*====================================================================
  Set delay time:x*1us
====================================================================*/
void DelayUs(uint us)
{
    while(us--);
}
/*====================================================================
  Set delay time:x*1ms
====================================================================*/
void DelayMs(uint Ms)
{
  uint i,TempCyc;
  for(i=0;i<Ms;i++)
  {
    TempCyc = 250;
    while(TempCyc--);
  }
 }
/*========================================================
 Timer 0 interrupt service routine  PWM1
========================================================*/
void timer0() interrupt 1 
{  
	unsigned int count=0;
	count = (unsigned int)PWM1_value*10;
	MOTOR1_PWM_S = ~MOTOR1_PWM_S;
	if(0==MOTOR1_PWM_S)
	{
	   count = 0xffff-count;
	   if(MOTOR1_SW_S==1)
	   {
	    if(0<PWM1_value)
	      PWM1_OUT = 0;     
	   }
	   else PWM1_OUT = 1;   
	}
	else
	{
	  count = 0xffff-(1000-count);
	  if(100>PWM1_value)
      PWM1_OUT = 1;    
	}
	TH0=count/256;
    TL0=count%256;
    TR0=1 ;
}

/*========================================================
//Buzzer subroutine
========================================================*/
void Beep()     
{
    unsigned char i  ;
    for (i=0  ;i<100  ;i++)
    {
        DelayUs(100);
        BEEP=!BEEP;         //Beep negated
    } 
    BEEP=1  ;              //Close buzzer
} 
/*========================================================
//PWM Initialization routine
========================================================*/
void PWM_INIT(void)
{
    TMOD=0x11;
    TH0=0xff ;         
    TL0=0 ;       
    EA=1;
    ET0=1;
    TR0=1 ;

	PWM1_OUT = 0;  //Close PWM

	//Motor direction forward
    MOTOR1_DR1 = 1;
	MOTOR1_DR2 = 0;

}

/*========================================================
//Key scan routine
========================================================*/
void KEY_SCAN(void)
{

         //Start or stop
		 if((0== KEY_M1_SW)&&(0==KEY_M1_SW_S) )
		 {
		    DelayMs(50);
		    KEY_M1_SW_S = 1;
			MOTOR1_SW_S = ~MOTOR1_SW_S;
				  
		  }
		  if((1== KEY_M1_SW)&&(1==KEY_M1_SW_S))  KEY_M1_SW_S = 0;
			  
		  //Forward or reverse
		  if((0== KEY_M1_DR)&&(0==KEY_M1_DR_S ) )
		 {
		    DelayMs(50);
		    KEY_M1_DR_S = 1;
			MOTOR1_DR_S  = ~MOTOR1_DR_S ;

			if(0==MOTOR1_DR_S) //Motor Forward
			{
			   MOTOR1_DR1 = 1;
	           MOTOR1_DR2 = 0;
		
			}
			else //Motor reversal
			{
			     MOTOR1_DR1 = 0;
	             MOTOR1_DR2 = 1;
			}		  
		  }
		  if((1== KEY_M1_DR)&&(1==KEY_M1_DR_S))  KEY_M1_DR_S= 0;

		   //PWM1+
		  if(KEY_M1_ADD==0)
      	  {
			 if(PWM1_value<100)
			 {
               PWM1_value++ ; 
          	 }
			  if(PWM1_value==100)Beep();//PWM=100% Click buzzer
     	  }
  
          //PWM1-
		  if(KEY_M1_SUB==0)
      	  {
			 if(PWM1_value>0)
			 {
               PWM1_value-- ;
          	 }

			 if(PWM1_value==0)Beep();//PWM = 0% buzzer look
		  }

//================================================================================
		  if((KEY_M1_ADD==0)||(KEY_M1_SUB==0))
		  {
			   if(KeySpeedup<30)KeySpeedup++;   //Key Acceleration
		   }
		   else	KeySpeedup = 0;

		  DelayMs(31-KeySpeedup); 
}

/*=========================== The main program=============================*/
void main(void)
{   
   PWM_INIT();

   while(1)
   {	
	  KEY_SCAN();	    // Key scan

   }
}
